; Because this stuff (disassembly) is tricky, I'll explain my initial
; attempt at it. We have the mnemonic table (mnemtable.asm) with the
; opcodes and names of all the instructions. Instructions like
; 'anda' which require only one byte for the instruction are 
; called inherent. So the disassembler checks for inherent codes
; first because they're easy. Actually, it checks for "page 2
; inherents" first because they're easy to separate from all 
; other instructions (because they all have two bytes, the first of
; which is the "page 2 prebyte" -- $18.) So we can easily look for
; page2 inherent instructions first. If the opcode we have isn't 
; found among the inherent or p2inh instructions, then its probably
; a relative (branch) or general (like ldaa) instruction. The relative
; instructions are easiest to find and deal with so we check for them
; next. The general ("gen") instructions are those like "ldaa" that
; have 3/4 opcodes associated with the one instruction. To find out 
; is our opcode is a general one, we need to AND it with a constant
; to get to it's "base" opcode. (ldaa, for example, can be $a6 so we
; AND it with $cf to get its base opcode, $86.) This AND'ing thing 
; seems to work very well. (Or maybe it doesn't.) There is another
; strange class of codes that we need to deal with (marked "BAD" in
; mnemtable.asm) but I don't know how to do that yet.


; -------------= search mnemonic table =--------------
; A=opcode to search for
; B=class 
; returns: Y=addr of name of opcode
;          B=$ff on error, "real" class on exit
; ----------------------------------------------------
searchmnem:
	ldy		#mnemtable		; Y points to the table
	pshb
	ldab		#$04			; 
	aby					; we need to add 4 to Y to get it to the class field
	pulb					; restore B
sm_loop1:
	; check to see if its the right class
	cmpb		0,y
	beq		sm_same_class
	orab		#%10000000		; check for the long bit instructions, too
	cmpb		0,y
	beq		sm_same_class		; if yes, check the opcode for similarity
	andb		#%01111111		; remove the long bit
		
	pshb		
	ldab		#$06			; skip to the next mnemonic
	aby					;
	pulb					; restore B (class)

sm_chk_end:
	cpy		#lastclass		; have we reached the end of the table?
	beq		sm_error_exit		; yes, exit with error

	bra		sm_loop1		; and continue the search

sm_same_class:
	; the class of the opcode we're looking for
	; is the same as this opcode so check the
	; opcodes -- see if they're similar
	iny					; move Y to opcode in list
	cmpa		0,y			; opcodes the same?
	beq		sm_opcode_found		; yes, we found it so return success!
	
	pshb					; if it isn't the same opcode,
	ldab		#$05			; add 5 (to get past name of next opcode)
	aby					;
	pulb					;
	
	bra		sm_chk_end		; and then jump to continue checking

sm_opcode_found:
	; we found the opcode so we need to subtract 5 from Y
	; to get it to point at the name of this opcode
	; we also need to load into B the "real" class of this
	; code
	ldab		-1,y			; loads the class into B

	xgdy					; X <=> D
	subd		#$05			; take 5 off 	
	xgdy					; put it back	

	rts					; Y is set and B is still the class so
						; we can just return
sm_error_exit:
	ldab		#$ff			; error code
	rts
	
	
; --------------= disassembly! =---------------
; in: X = addr of first byte in instruction
; out: X = addr of next instruction!
; ---------------------------------------------

rr_list	dc.b		'xysp'

disasm:
	stx		tempw			; save the addr of the instruction

	ldaa		0,x			; A=first byte of instruction
	
	cmpa		#PAGE2			; is it a page 2 instruction?
	beq		da_page2_instr		; yes, go check for page2 instructions

	jsr		scan_spec_instr		; check if its a "special" instruction!
	cmpb		#$ff			; $ff means it wasn't a special instruction
	lbne		da_rts			; 

	jsr		scan_spec2_instr	; check for the other special class
	cmpb		#$ff
	lbne		da_rts

						; otherwise we'll check all of the inh instructions
	; look for inh instructions
	ldab		#INH			; class
	jsr		searchmnem		; scan the table for this mnemonic (returns ptr to
						; name in Y, $ff in B on error)
	cmpb		#$ff			; error (means opcode not found)?
	beq		da_scan_rel		; yes, check for relative instructions (REL)

	jsr		print_mnem
	inx					; move X to next instruction
	rts
	
da_page2_instr:
	inx					; we need the next op -- the real opcode
	ldaa		0,x			; get it

	ldab		#P2INH			; look for a page2 inherent op
	jsr		searchmnem		; search for P2INH mnemonic

	cmpb		#$ff			; error (mnemonic not found)
	bne		da_p2_print		; if found, print the mnemonic

	ldab		#P2LREL			; look for the long relative branches
	jsr		searchmnem		; search for P2LREL mnemonic
	ldx		tempw
	inx	
	inx	
	inx
	inx					; move to next instruction
	rts

da_p2_print:
	jsr		print_mnem		; print it
	
	ldx		tempw
	inx
	inx
	inx					; increment X to the next instruction
	rts

da_scan_rel:
	ldx		tempw			; restore the addr of the instruction
	ldaa		0,x			; and re-read the opcode

	ldab		#REL			; look for relative (rel) instructions
	jsr		searchmnem		; do it

	cmpb		#$ff			; error (opcode not found)
	beq		da_scan_gen		; yes, go to general instructions

	jsr		print_mnem		; print the mnemonic
	inx					; get the relative part at
	ldaa		0,x			; the next byte in the object code

	psha
	ldaa		#' '
	jsr		put_char
	pula
	jsr		print_number		; print the relative part (fix this later?)
						; we'll need to do ADDR+relative part. print that.
	inx					; move X to next instruction
	rts


da_scan_gen:
	; now the general one's need to be AND'd with
	; 0xCF to get the "base opcode". like there is
	; an LDAA with an opcode of $A6. But the base 
	; is $86 (the one in my table) so we do
	; $a6 AND $cf = $86. See?

	; after finding the general instruction, we need to 
	; evaluate which "flavor" of that instruction it is.
	; ldaa, for example, has 4 "flavors" and the "$A6"
	; flavor has 5 different types!


	ldx		tempw			; get teh addr of the op again
	ldaa		0,x			; and the op
	
	anda		#$cf			; AND it with CF, remember?
	ldab		#GEN			; we're looking for general instructions

	jsr		searchmnem		; look for it

	cmpb		#$ff			; couldn't find it?
	lbeq		da_scan_gen2		; nope

	jsr		print_mnem

	; lets decode the type of general instruction
	pshb
	psha
	pulb					; transfer A to B
	ldaa		0,x			; get the original instruction
	
	sba					; A-B => A
	asra					; we need to shift A right 4 to get
	asra					; a usable number
	asra					; 0=immediate, 1=direct, 2=extended, 3=index
	asra
	
	pulb
	cmpa		#$00	
	beq		da_gen_imm		; immediate
	cmpa		#$01			
	beq		da_gen_dir		; direct
	cmpa		#$03			
	beq		da_gen_extend		; extended
	cmpa		#$02			
	beq		da_gen_index		; indexed	


da_gen_imm:
	; immediate is easy: the next byte is the value
	; but we have to check B to make sure it isn't
	; a long immediate instructions (ldd, ldy, ldx)
	ldaa		1,x
	psha
	ldaa		#'#'
	jsr		put_char
	pula
	jsr		print_number
	andb		#%10000000		; check for long instruction
	beq		da_imm_end		; if zero, then it wasn't a long inst

	ldaa		2,x
	jsr		print_number	

	inx					; return with X pointing to the next instr
da_imm_end:
	inx
	inx

	rts		

da_gen_dir:
	; direct is easy, too: next byte is address
	ldaa		1,x
	jsr		print_number

	ldx		tempw			; load the addr of the op
	inx
	inx					; increment to the next instr

	rts

da_gen_extend:
	; extended is easy: next two bytes are address
	ldd		1,x
	xgdx
	jsr		print_address
	xgdx
	
	inx					; advance to next instr
	inx
	inx

	rts

da_gen_index:
	; index is tough. see p. 3-6 of the CPU12 manual for
	; info on how to decode the 'xb' or instruction
	; post-byte to get the index info.
	ldx		tempw			; get the addr of the instruction again
	inx					; skip to the post-byte
	ldaa		0,x			; get the post-byte

	; first we check for 5-bit constant offset
	anda		#%00100000		; if this returns zero, it is
	beq		da_index_5bit		; 

	; now we check for the 9,16-bit constant offset
	ldaa		0,x
	anda		#%11100100
	cmpa		#%11100000		; if its equal to this, then its either 
						; 9 or 16 bit offset
	bne		da_cont			; continue with checking

	ldaa		0,x		
	anda		#%00000011		; if those two bits are set, its 16
	lbeq		da_index_16		; if its 16 bit const offset
	lbra		da_index_9		; if its 9 bit const offset

da_cont:
	; check for accumulator offset (A,B,D, or D indexed indirect)
	ldaa		0,x
	anda		#%11100100
	cmpa		#%11100100
	lbeq		da_accum_offset

	; if that fails, then its auto-pre/post increment/decrement
	bra		da_auto_inc_dec

da_index_5bit:
	ldx		tempw
	inx					; to get to the post-byte
	ldaa		0,x
	anda		#%11000000		; get the index reg involved
	psha
	ldaa		0,x
	anda		#%00011111		; get the offset
	pulb
	asrb					; faster to shift than divide!
	asrb
	asrb
	asrb
	asrb
	asrb
	; now A is the offset (signed) and B is the register
	; print the offset
	jsr		print_number
	ldaa		#','
	jsr		put_char
	; find the register and print
	pshx
	ldx		#rr_list
	pshb
	pula
	abx	
	ldaa		0,x
	jsr		put_char
	pulx	

	inx					; increment X to the next instr

	rts

da_index_9:
	ldx		tempw			; get addr of the op
	ldaa		1,x			; get the xb
	anda		#%00011000		; 
	asra
	asra
	asra					; extract the register code
	ldx		#rr_list
	psha
	pulb
	abx
	ldaa		0,x			; get the register character
	psha
	ldaa		2,x
	jsr		print_number
	ldaa		#','
	jsr		put_char
	pula
	jsr		put_char
	
	ldx		tempw
	inx					; increment X to next instr
	inx
	inx
	rts	

da_scan_gen2:
	; this could be combined with da_scan_gen
	; but I think this makes it clearer. There
	; is a block of general STORE instructions
	; (staa, stab, std, sty, stx, sts) that need 
	; to be AND'd with $cf and 16 added so that
	; a $6a (staa 0,y) becomes $5a ($6a AND $cf+$10)
	adda		#$10			; add 16
	ldab		#GEN
	jsr		searchmnem		; do the search
	
	cmpb		#$ff			; didn't find it?
	beq		da_done			; nope

	jsr		print_mnem

	; found it
	; do like in gen where we figure out which 'version' of
	; the instruction it is
	psha
	pulb
	ldaa		0,x
	sba					; A-B => A
	asra
	asra
	asra
	asra

	cmpa		#$00
	lbeq		da_gen_dir		; direct, not immediate
	cmpa		#$02
	lbeq		da_gen_extend		; indexed
	
	; otherwise its indexed -- hard!
	lbra		da_gen_index

	rts


da_done:
da_index_16:
da_accum_offset:
da_auto_inc_dec:
	ldab		#$ff
da_rts:
	rts

; scan_spec2_instr looks for SPEC2 instructions and decodes them
scan_spec2_instr:
	psha
	ldx		tempw
	ldaa		0,x			; get the instr opcode

	ldab		#SPEC2	
	jsr		searchmnem		; look for it

	cmpb		#$ff			; if $ff it wasn't found
	bne		ssi_found		; otherwise it was found

	suba		#$10			; all SPEC2 instructions have two 
	ldab		#SPEC2			; codes: n and n-$10
	jsr		searchmnem
	
	cmpb		#$ff
	bne		ssi_found2		; not equal to $ff then it was found
	
	beq		ssi_err_done		; otherwise it was not found at all -- ret error

ssi_found2:
	adda		#$10			; re-add $10 to opcode
ssi_found:
	; print the opcode
	jsr		print_mnem

	; we need to get the "base" opcode
	ldab		#$05
	aby					; add 5 to Y to get base op
	ldab		0,y

	sba					; A-B => A
	beq		ssi_index_op		; if A=0, the op we have is the "base op" (low one)
						; which is really the indexed op
ssi_ext_op:
	; otherwise its the extended version with the next two
	; bytes being the 16 bit addr
	ldd		1,x			; load D with the 16 bit addr
	xgdx
	jsr		print_address
	ldx		tempw
	inx					; advance to the next instr
	inx
	inx
	bra		ssi_done_good

ssi_index_op:
	jsr		da_gen_index		; use pre-existing code!
	bra		ssi_done_good

ssi_err_done:
	ldab		#$ff
ssi_done_good:
	pula
	rts


; da_scan_spec scans for special instructions like JSR and deals with 'em
scan_spec_instr:
	psha		

	ldx		tempw
	ldaa		0,x
		
	; check for JSR which has opcodes $15, $16, $17
	ldab		jsr_op
	
	deca
	cba
	beq		dss_is_jsr

	deca		
	cba	
	beq		dss_is_jsr
	
	bra		dss_done_bad

dss_is_jsr:
	ldy		#jsr_mnem
	jsr		print_mnem
	ldaa		0,x			; get the op
	cmpa		#$17
	beq		dss_jsr_dir		; direct
	cmpa		#$16			
	beq		dss_jsr_extend		; extended
	
	; otherwise indirect

dss_jsr_dir:
	ldaa		1,x			; load the direct byte
	jsr		print_number
	inx
	inx					; increment X to next instr
	bra		dss_done_good		; all done

dss_jsr_extend:	
	pshx
	ldd		1,x			; load D with extended word	
	xgdx
	jsr		print_address
	pulx
	inx					; increment X to next instr
	inx
	inx
	bra		dss_done_good

dss_done_bad:
	ldab		#$ff
dss_done_good:
	pula
	rts



; print_mnem just prints the 4 characters at Y
print_mnem:
	pshd
	ldaa		#13
	jsr		put_char
	ldaa		#10
	jsr		put_char
	ldaa		0,y	
	jsr		put_char
	iny
	ldaa		0,y	
	jsr		put_char
	iny
	ldaa		0,y	
	jsr		put_char
	iny
	ldaa		0,y	
	jsr		put_char
	iny
	ldaa		#32
	jsr		put_char
	puld
	rts


; we need the mnemonics table, duh
#include "mnemtable.asm"

