;#ifndef	__SERIAL_ASM__
;#define	__SERIAL_ASM__

SC0BDH	equ	$00C0	; SCI baud rate control registers
SC0BDL	equ	$00C1	;

SC0CR1	equ	$00C2	; SCI Control Register 1
SC0CR2	equ	$00C3	; SCI Control Register 2
SC0SR1	equ	$00C4	; SCI Status Register 1
SC0SR2	equ	$00C5	; SCI Status Register 2
SC0DRH	equ	$00C6	; SCI Data Register High
SC0DRL	equ	$00C7	; SCI Data Register Low

; ---------- subroutines ------------

tc_check:
	psha
tc_2:	
     ldaa	SC0SR1
	anda	#%01000000
	beq		tc_2
	pula
	rts	

init_serial_tx:
	; set 9600 baud
	ldaa	#$00
	staa	SC0BDH
	ldaa	#$1a			; this sets 9600
	staa	SC0BDL			; takes effect here
	; turn on the transmitter only
	ldaa	#%00001000
	staa	SC0CR2
	rts

init_serial_tx_rx:
	; set 9600 baud
	ldaa	#$00
	staa	SC0BDH
	ldaa	#$1a		; this sets 9600
	staa	SC0BDL		; takes effect here
	; turn on the transmitter and receiver
	ldaa	#%00001100
	staa	SC0CR2
	rts

; get_char 
; returns in the A register a character from the
; serial receiver IF ONE IS AVAILABLE
; does not block
; messes with A
get_char:
	ldaa	SC0SR1
	anda	#%00100000
	beq		no_char_rx
	ldaa	SC0DRL
no_char_rx:
	rts

; get_char_wait
; returns in the A register a character from the
; serial receiver and waits for it -- blocks
; messes with A
get_char_wait:
	ldaa	SC0SR1
	anda	#%00100000
	beq		get_char_wait
	ldaa	SC0DRL
	bsr		put_char		; echo it
	rts

get_char_wait_noecho:
	ldaa	SC0SR1
	anda	#%00100000
	beq	get_char_wait_noecho
	ldaa	SC0DRL
	rts


; put_char
; sends the character in A register out
; the serial port if possible --
; it checks the TC flag and blocks until
; it is set
put_char:
	psha	               ; save A because tc_check messes it up
	bsr		tc_check	
	pula	
	staa	SC0DRL
	rts

; print_string takes the address of the 
; string to be printed in the Y register 
; and sends characters to the serial port
; until a null (0) character
print_string:
	ldaa	0,y			; load the character
	beq	done_ps		; if its a zero, end
	iny				; increment the Y register
	bsr	tc_check		; wait until the transmiter is done
	staa	SC0DRL		; send it to the transmitter
	bra	print_string	; start over
done_ps:	
	rts				; go back

; print number
; prints in ASCII hex the number in
; register A
print_number:
     psha
     anda #%11110000     ; get upper nibble
     lsra                ; shift it right to get the nibble
     lsra
     lsra
     lsra		
     bsr  pn_doit        ; change A and print it
pn_next:
     pula                ; get the next nibble
     anda #%00001111     ; mask to do so
     bsr  pn_doit        ; change A and print it
     rts
pn_doit:
     cmpa #$09           ; is it greater than 9?
     bgt  pn_abcdef      ; if so, print a character
     adda #48            ; if not, print a number starting at 48 ASCII
     bra  pn_print       ;
pn_abcdef:     
     adda #55             ; if so, print a letter starting at 55 = 65-10 
pn_print:
     bsr  put_char       ; print it
     rts

; print_address prints the address in
; X
print_address:
	pshd
	pshx
	xgdx	; put X into D -- A and B
	bsr		print_number		; prints whats in A -- MSB
	pshb
	pula	; get b into A
	bsr		print_number		; prints whats in B -- LSB
	pulx						; restore X and D
	puld
	rts

;#endif


	
