SC0BDH	equ	$00C0	; SCI baud rate control registers
SC0BDL	equ	$00C1	;

SC0CR1	equ	$00C2	; SCI Control Register 1
SC0CR2	equ	$00C3	; SCI Control Register 2
SC0SR1	equ	$00C4	; SCI Status Register 1
SC0SR2	equ	$00C5	; SCI Status Register 2
SC0DRH	equ	$00C6	; SCI Data Register High
SC0DRL	equ	$00C7	; SCI Data Register Low
COPRST	equ	$0017	; COP Arm/Reset COP Register
COPCTL	equ	$0016	; COP Control Register

	org	$FFFE

RESET_VECTOR	dw	$E000

	org	$E000

; turn off the COP watchdog
; COPCTL can only be written once in normal
; mode so we use it to turn off the COP
; watchdog timer rate generator!
cop_off:
	ldaa	#$00
	staa	COPCTL

	lds		#$0bff	; init the stack pointer

; set the baud rate 9600
set_baud:
	ldaa	#$00
	staa	SC0BDH
	ldaa	#$1a	; this sets 9600
	staa	SC0BDL	; takes effect here

; next, turn on the transmitter
	ldaa	#%00001000
	staa	SC0CR2
	
; loop while checking the TC flag in SC0SR1
check_tc:
	ldaa	SC0SR1
	anda	#%01000000
	beq	check_tc

; otherwise, send data to the data register

	ldaa	#'b'
	staa	SC0DRL
	bsr	tc_check
do:
	ldy	#message		; test the new code
	bsr	print_string	; ditto
	bra	do
	
message	dc.b	"Hello World!",13,10,0	

; ---------- subroutines ------------

tc_check:
	ldaa	SC0SR1
	anda	#%01000000
	beq	tc_check
	rts	

init_serial:
	; set 9600 baud
	ldaa	#$00
	staa	SC0BDH
	ldaa	#$1a		; this sets 9600
	staa	SC0BDL		; takes effect here
	; turn on the transmitter (only)
	ldaa	#%00001000
	staa	SC0CR2
	rts

; print_string takes the address of the 
; string to be printed in the Y register 
; and sends characters to the serial port
; until a null (0) character
print_string:
	ldaa	0,y			; load the character
	beq	done_ps			; if its a zero, end
	iny					; increment the Y register
	staa	SC0DRL		; send it to the transmitter
	bsr	tc_check		; wait until the transmiter is done
	bra	print_string	; start over
done_ps:	
	rts					; go back